package compbio.data.sequence;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;

@XmlAccessorType(XmlAccessType.FIELD)
public class ScoreManager {

	public static final String SINGLE_ENTRY_KEY = "Alignment";

	private List<ScoreHolder> seqScores;

	private ScoreManager() {
		// Default JAXB constructor
	}

	private ScoreManager(String id, Set<Score> data) {
		seqScores = new ArrayList<ScoreManager.ScoreHolder>();
		seqScores.add(new ScoreHolder(id, data));
	}

	private ScoreManager(Map<String, Set<Score>> data) {
		List<ScoreHolder> seqScores = new ArrayList<ScoreHolder>();
		for (Map.Entry<String, Set<Score>> singleSeqScores : data.entrySet()) {
			seqScores.add(new ScoreHolder(singleSeqScores.getKey(),
					singleSeqScores.getValue()));
		}
		this.seqScores = seqScores;
	}

	public static ScoreManager newInstance(Map<String, Set<Score>> data) {
		return new ScoreManager(data);
	}

	public static ScoreManager newInstanceSingleScore(
			Map<String, Score> seqScoresMap) {
		Map<String, Set<Score>> multipleScoresMap = new HashMap<String, Set<Score>>();
		for (Map.Entry<String, Score> seqScore : seqScoresMap.entrySet()) {
			Set<Score> scores = new HashSet<Score>();
			scores.add(seqScore.getValue());
			multipleScoresMap.put(seqScore.getKey(), scores);
		}
		return new ScoreManager(multipleScoresMap);
	}

	public static ScoreManager newInstanceSingleSequence(Set<Score> data) {
		return new ScoreManager(ScoreManager.SINGLE_ENTRY_KEY, data);
	}

	public Map<String, Set<Score>> asMap() {
		Map<String, Set<Score>> seqScoresMap = new HashMap<String, Set<Score>>();
		for (ScoreHolder sch : this.seqScores) {
			Set<Score> oldValue = seqScoresMap.put(sch.id, sch.scores);
			if (oldValue != null) {
				throw new IllegalStateException(
						"Cannot represent this ScoreManager instance "
								+ "as a Map as it contains duplicated keys: "
								+ sch.id);
			}
		}
		return seqScoresMap;
	}

	public Set<Score> asSet() {
		if (seqScores.size() == 0 || seqScores.size() > 1) {
			throw new IllegalStateException(
					"This ScoreManager has no or multiple sequence entries and thus "
							+ "cannot be represented as a Set. Number of entries are: "
							+ seqScores.size());
		}
		ScoreHolder sch = seqScores.get(0);
		return sch.scores;
	}

	public int getNumberOfSeq() {
		return seqScores.size();
	}

	public ScoreHolder getAnnotationForSequence(String seqId) {
		for (ScoreHolder sch : seqScores) {
			if (sch.id.equals(seqId)) {
				return sch;
			}
		}
		return null;
	}

	public void writeOut(OutputStream outStream) throws IOException {
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(
				outStream));
		for (ScoreHolder oneSeqScores : seqScores) {
			oneSeqScores.writeOut(outStream);
		}
		writer.flush();
	}

	@XmlAccessorType(XmlAccessType.FIELD)
	public static class ScoreHolder {

		public String id;
		public Set<Score> scores;

		private ScoreHolder() {
			// JAXB Default constructor should not be used otherwise
		}

		ScoreHolder(String id, Set<Score> scores) {
			this.id = id;
			this.scores = scores;
		}

		public void writeOut(OutputStream outStream) throws IOException {
			BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(
					outStream));
			writer.write(">" + id + "\n");
			Score.write(scores, outStream);
		}

		public Score getScoreByMethod(Enum<?> method) {
			for (Score sc : scores) {
				if (method == sc.getMethod()) {
					return sc;
				}
			}
			return null;
		}

		public int getNumberOfScores() {
			return scores.size();
		}
	}
}
